@import UIKit;
#import "RGLDocumentReader.h"
@import DocumentReader;

@implementation RGLDocumentReader

static NSNumber * _databasePercentageDownloaded;

+ (NSNumber*)databasePercentageDownloaded{ return _databasePercentageDownloaded; }
+ (void)setDatabasePercentageDownloaded:(NSNumber *)number { _databasePercentageDownloaded = number; }


- (void) startRFIDReader:(CDVInvokedUrlCommand*)command{
    if([self.docReader isDocumentReaderIsReady])
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.docReader startRFIDReaderFromPresenter:[[[UIApplication sharedApplication] keyWindow] rootViewController] completion:[self getCompletion: command]];
        });
    else
        [self result:@"document reader not ready" command:command];
}

- (void) initializeReader:(CDVInvokedUrlCommand*)command{
    self.docReader = RGLDocReader.shared;
    [self.docReader initializeReader:[[command arguments] objectAtIndex:0] completion:[self getInitCompletion: command]];
}

- (void) initializeReaderWithDatabasePath:(CDVInvokedUrlCommand*)command{
    self.docReader = RGLDocReader.shared;
    [self.docReader initializeReader:[[NSData alloc] initWithBase64EncodedString:[[command arguments] objectAtIndex:0] options:0] databasePath:[[command arguments] objectAtIndex:1] completion:[self getInitCompletion: command]];
}

- (void) prepareDatabase:(CDVInvokedUrlCommand*)command{
    self.docReader = RGLDocReader.shared;
    [self.docReader prepareDatabase:[[command arguments] objectAtIndex:0] progressHandler:[self getProgressHandler:command] completion:[self getPrepareCompletion: command]];
}

- (void) removeDatabase:(CDVInvokedUrlCommand*)command{
    [self.docReader removeDatabase:^(BOOL successful, NSString * _Nullable error) {
        if (successful)
            [self result:@"database removed" command:command];
        else
            [self error:[NSString stringWithFormat:@"%@/%@", @"database removal failed: ", error] command:command];
    }];
}

- (void) cancelDBUpdate:(CDVInvokedUrlCommand*)command{
    [self.docReader cancelDBUpdate];
    [self result:@"" command:command];
}

-(void) runAutoUpdate:(CDVInvokedUrlCommand*)command{
    self.docReader = RGLDocReader.shared;
    [self.docReader runAutoUpdate:[[command arguments] objectAtIndex:0] progressHandler:[self getProgressHandler:command] completion:[self getPrepareCompletion: command]];
}

- (void) showScanner:(CDVInvokedUrlCommand*)command{
    if([self.docReader isDocumentReaderIsReady]){
        dispatch_async(dispatch_get_main_queue(), ^{
            [[UIApplication sharedApplication] setStatusBarStyle:UIStatusBarStyleLightContent animated:YES];
            UIViewController *currentViewController = [[[UIApplication sharedApplication] keyWindow] rootViewController];
            [self.docReader showScanner:currentViewController completion:[self getCompletion: command]];
        });
    }else
        [self result:@"document reader not ready" command:command];
}

- (void) recognizeImage:(CDVInvokedUrlCommand*)command{
    [self recognizeImageWith:command :[[command arguments] objectAtIndex:0] :false];
}

- (void) recognizeImages:(CDVInvokedUrlCommand*)command{
    if([self.docReader isDocumentReaderIsReady]){
        dispatch_async(dispatch_get_main_queue(), ^{
            NSArray* input = [[command arguments] objectAtIndex:0];
            NSMutableArray<UIImage*>* images = [[NSMutableArray init] alloc];
            for(__strong NSMutableString* base64 in input){
                if(![[base64 substringToIndex:10] isEqualToString:@"data:image"])
                    base64 = [NSMutableString stringWithFormat: @"%@%@", @"data:image/jpeg;base64,", base64];
                [images addObject:[UIImage imageWithData:[NSData dataWithContentsOfURL:[NSURL URLWithString:base64]]]];
            }
            [self.docReader recognizeImages:images completion:[self getCompletion: command]];
        });
    }else
        [self result:@"document reader not ready" command:command];
}

- (void) recognizeImageWithCameraMode:(CDVInvokedUrlCommand*)command{
    [self recognizeImageWith:command :[[command arguments] objectAtIndex:0] :[[command arguments] objectAtIndex:1]];
}

- (void) recognizeImageWith:(CDVInvokedUrlCommand*)command :(NSMutableString*)base64 :(BOOL)cameraMode{
    if([self.docReader isDocumentReaderIsReady]){
        if(![[[[command arguments] objectAtIndex:0] substringToIndex:10] isEqualToString:@"data:image"])
            base64 = [NSMutableString stringWithFormat: @"%@%@", @"data:image/jpeg;base64,", [[command arguments] objectAtIndex:0]];
        [self.docReader recognizeImage:[UIImage imageWithData:[NSData dataWithContentsOfURL:[NSURL URLWithString:base64]]] cameraMode:cameraMode completion:[self getCompletion: command]];
    }else
        [self result:@"document reader not ready" command:command];
}

- (void) setConfig:(CDVInvokedUrlCommand*)command{
        [RegulaConfig setConfig:[[command arguments] objectAtIndex:0] :self.docReader];
        [self result:@"" command:command];
}

- (void) getConfig:(CDVInvokedUrlCommand*)command{
    [self result:[[NSString alloc] initWithData:[NSJSONSerialization dataWithJSONObject:[RegulaConfig getConfig:self.docReader] options:NSJSONWritingPrettyPrinted error:nil] encoding:NSUTF8StringEncoding] command:command];
}

- (void) setRfidScenario:(CDVInvokedUrlCommand*)command{
        [RegulaConfig setRfidScenario:[[command arguments] objectAtIndex:0] : self.docReader.rfidScenario];
        [self result:@"" command:command];
}

- (void) getRfidScenario:(CDVInvokedUrlCommand*)command{
        [self result:[[NSString alloc] initWithData:[NSJSONSerialization dataWithJSONObject:self.docReader.rfidScenario.rfidScenarioDictionary options:NSJSONWritingPrettyPrinted error:nil] encoding:NSUTF8StringEncoding] command:command];
}

- (void) readRFID:(CDVInvokedUrlCommand*)command{
    if([self.docReader isDocumentReaderIsReady])
        [self.docReader readRFID:nil completion:[self getRFIDCompletion: command]];
    else
        [self result:@"document reader not ready" command:command];
}

- (void) stopRFIDReader:(CDVInvokedUrlCommand*)command{
    [self.docReader stopRFIDReader:^(){[self result:@"" command:command];}];
}

- (void) clearPKDCertificates:(CDVInvokedUrlCommand*)command{
    [self.docReader clearPKDCertificates];
    [self result:@"" command:command];
}

- (void) addPKDCertificates:(CDVInvokedUrlCommand*)command{
    NSArray* input = [[command arguments] objectAtIndex:0];
    NSMutableArray<RGLPKDCertificate*>* certificates = [[RGLPKDCertificate init] alloc];
    for(NSDictionary* certificateJSON in input)
        [certificates addObject:[JsonConstructor RGLPKDCertificateFromJson:certificateJSON]];
    [self.docReader addPKDCertificates:certificates];
    [self result:@"" command:command];
}

- (void) deinitializeReader:(CDVInvokedUrlCommand*)command{
    [self.docReader deinitializeReader];
    [self result:@"" command:command];
}

- (void) selectedScenario:(CDVInvokedUrlCommand*)command{
    [self result:[JsonConstructor generateScenario:self.docReader.selectedScenario] command:command];
}

- (void) stopScanner:(CDVInvokedUrlCommand*)command{
    [self.docReader stopScanner:^(){[self result:@"" command:command];}];
}

- (void) startNewSession:(CDVInvokedUrlCommand*)command{
    [self.docReader startNewSession];
    [self result:@"" command:command];
}

- (void) startNewPage:(CDVInvokedUrlCommand*)command{
    [self.docReader startNewPage];
    [self result:@"" command:command];
}

- (void) getDocumentReaderIsReady:(CDVInvokedUrlCommand*)command{
    [self result:[self.docReader isDocumentReaderIsReady] ? @YES : @NO command:command];
}

- (void) getAPIVersion:(CDVInvokedUrlCommand*)command{
    [self result:[[self.docReader version] api] command:command];
}

- (void) getCoreVersion:(CDVInvokedUrlCommand*)command{
    [self result:[[self.docReader version] core] command:command];
}

- (void) getCoreMode:(CDVInvokedUrlCommand*)command{
    [self result:[[self.docReader version] coreMode] command:command];
}

- (void) getDatabaseID:(CDVInvokedUrlCommand*)command{
    [self result:[[[self.docReader version] database] databaseID] command:command];
}

- (void) getDatabaseVersion:(CDVInvokedUrlCommand*)command{
    [self result:[[[self.docReader version] database] version] command:command];
}

- (void) getDatabaseDate:(CDVInvokedUrlCommand*)command{
    [self result:[[[self.docReader version] database] date] command:command];
}

- (void) getDatabaseDescription:(CDVInvokedUrlCommand*)command{
    [self result:[[[self.docReader version] database] databaseDescription] command:command];
}

- (void) getDatabaseCountriesNumber:(CDVInvokedUrlCommand*)command{
    [self result:[NSString stringWithFormat: @"%ld", (long)self.docReader.version.database.countriesNumber] command:command];
}

- (void) getDatabaseDocumentsNumber:(CDVInvokedUrlCommand*)command{
    [self result:[NSString stringWithFormat: @"%ld", (long)self.docReader.version.database.documentsNumber] command:command];
}

- (void) getLicenseExpiryDate:(CDVInvokedUrlCommand*)command{
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    [formatter setFormatterBehavior:NSDateFormatterBehaviorDefault];
    [formatter setDateStyle:NSDateFormatterShortStyle];
    [formatter setTimeStyle:NSDateFormatterNoStyle];
    [self result:[formatter stringFromDate:self.docReader.license.expiryDate] command:command];
}

- (void) getLicenseCountryFilter:(CDVInvokedUrlCommand*)command{
    if(self.docReader.license.countryFilter == nil)
        [self result:@"nil" command:command];
    else
        [self result:[[NSString alloc] initWithData:[NSJSONSerialization dataWithJSONObject:self.docReader.license.countryFilter options:NSJSONWritingPrettyPrinted error:nil] encoding:NSUTF8StringEncoding] command:command];
}

- (void) licenseIsRfidAvailable:(CDVInvokedUrlCommand*)command{
    [self result:self.docReader.license.isRfidAvailable ? @YES : @NO command:command];
}

- (void) getLicenseMessage:(CDVInvokedUrlCommand*)command{
    [self result:@"nil" command:command];
}

- (void) getDocumentReaderStatus:(CDVInvokedUrlCommand*)command{
    [self result:self.docReader.documentReaderStatus command:command];
}

- (void) getRfidSessionStatus:(CDVInvokedUrlCommand*)command{
    [self result:self.docReader.rfidSessionStatus command:command];
}

- (void) getCurrentScenario:(CDVInvokedUrlCommand*)command{
    [self result:self.docReader.processParams.scenario command:command];
}

- (void) getCameraSessionIsPaused:(CDVInvokedUrlCommand*)command{
    [self result:self.docReader.isCameraSessionIsPaused ? @YES : @NO command:command];
}

- (void) setCameraSessionIsPaused:(CDVInvokedUrlCommand*)command{
    self.docReader.cameraSessionIsPaused = [[command arguments] objectAtIndex:0];
    [self result:@"" command:command];
}

- (void) isRFIDAvailableForUse:(CDVInvokedUrlCommand*)command{
    [self result:self.docReader.isRFIDAvailableForUse ? @YES : @NO command:command];
}

- (void) getSessionLogFolder:(CDVInvokedUrlCommand*)command{
    [self result:self.docReader.processParams.sessionLogFolder command:command];
}

- (void) getAvailableScenarios:(CDVInvokedUrlCommand*)command{
    NSMutableArray *availableScenarios = [[NSMutableArray alloc] init];
    for(RGLScenario *scenario in [self.docReader availableScenarios])
        [availableScenarios addObject:[JsonConstructor generateScenario:scenario]];
    [self result:[[NSString alloc] initWithData:[NSJSONSerialization dataWithJSONObject:availableScenarios options:NSJSONWritingPrettyPrinted error:nil] encoding:NSUTF8StringEncoding] command:command];
}

-(RGLDocumentReaderInitializationCompletion)getInitCompletion:(CDVInvokedUrlCommand*)command{
    return ^(BOOL successful, NSString * _Nullable error ) {
        if (successful)
            [self result:@"init complete" command:command];
        else
            [self error:[NSString stringWithFormat:@"%@/%@", @"init failed: ", error] command:command];
    };
}

-(void (^_Nullable)(NSProgress * _Nonnull progress))getProgressHandler:(CDVInvokedUrlCommand*)command{
    return ^(NSProgress * _Nonnull progress) {
        if(RGLDocumentReader.databasePercentageDownloaded != [NSNumber numberWithDouble:progress.fractionCompleted * 100]){
            CDVPluginResult* result = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString:[NSString stringWithFormat:@"%@%.1f%@", @"Downloading database: ", progress.fractionCompleted * 100, @"%"]];
            [result setKeepCallbackAsBool:YES];
            [self.commandDelegate sendPluginResult:result callbackId:command.callbackId];
            [RGLDocumentReader setDatabasePercentageDownloaded:[NSNumber numberWithDouble:progress.fractionCompleted * 100]];
        }
    };
}

-(RGLDocumentReaderPrepareCompletion _Nonnull)getPrepareCompletion:(CDVInvokedUrlCommand*)command{
    return ^(BOOL successful, NSString * _Nullable error) {
        if (successful)
            [self result:@"database prepared" command:command];
        else
            [self error:[NSString stringWithFormat:@"%@/%@", @"database preparation failed: ", error] command:command];
    };
}

-(RGLDocumentReaderCompletion _Nonnull)getCompletion:(CDVInvokedUrlCommand*)command{
    return ^(RGLDocReaderAction action, RGLDocumentReaderResults * _Nullable results, NSString * _Nullable error) {
        switch (action) {
            case RGLDocReaderActionCancel: {
                [self result:@"Cancelled by user" command:command];
                break;
            }
                
            case RGLDocReaderActionComplete: {
                [self result:[JsonConstructor resultsToJsonString:results] command:command];
                break;
            }
                
            case RGLDocReaderActionError: {
                [self error:[NSString stringWithFormat:@"%@/%@", @"error: ", error] command:command];
                break;
            }
                
            case RGLDocReaderActionProcess:
            case RGLDocReaderActionMorePagesAvailable: {
                break;
            }
        }
    };
}

-(RGLRFIDProcessCompletion _Nonnull)getRFIDCompletion:(CDVInvokedUrlCommand*)command{
    return ^(RGLRFIDCompleteAction action, RGLDocumentReaderResults * _Nullable results, NSString * _Nullable error, RGLRFIDErrorCodes errorCode) {
        switch (action) {
            case RGLRFIDCompleteActionCancel: {
                [self result:@"Cancelled by user" command:command];
                break;
            }
                
            case RGLRFIDCompleteActionComplete: {
                [self result:[JsonConstructor resultsToJsonString:results] command:command];
                break;
            }
                
            case RGLRFIDCompleteActionError: {
                [self error:[NSString stringWithFormat:@"%@/%@", @"error: ", error] command:command];
                break;
            }
            case RGLRFIDCompleteActionSessionRestarted: {
                break;
            }
        }
    };
}

- (void) result:(NSString*)message command:(CDVInvokedUrlCommand*)command{
    [self.commandDelegate sendPluginResult:[CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsString:message] callbackId:command.callbackId];
}

- (void) error:(NSString*)error command:(CDVInvokedUrlCommand*)command{
    [self.commandDelegate sendPluginResult:[CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:error] callbackId:command.callbackId];
}

@end
